<?php
/**
 * A database result class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class DatabaseResult extends BaseFrameworkSystem implements SearchableResult, SeekableIterator {
	// Exception constants
	const EXCEPTION_INVALID_DATABASE_RESULT = 0x0b0;

	/**
	 * Current position in array
	 */
	private $currentPos = -1;

	/**
	 * Current row
	 */
	private $currentRow = null;

	/**
	 * Result array
	 */
	private $resultArray = array();

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("Database result");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an instance of this result by a provided result array
	 *
	 * @param	$resultArray		The array holding the result from query
	 * @return	$resultInstance		An instance of this class
	 */
	public final static function createDatabaseResult (array $resultArray) {
		// Get a new instance
		$resultInstance = new DatabaseResult();

		// Set the result array
		$resultInstance->setResultArray($resultArray);

		// Return the instance
		return $resultInstance;
	}

	/**
	 * Setter for result array
	 *
	 * @param	$resultArray	The array holding the result from query
	 * @return	void
	 */
	protected final function setResultArray (array $resultArray) {
		$this->resultArray = $resultArray;
	}

	/**
	 * "Iterator" method next() to advance to the next valid entry. This method
	 * does also check if the result is invalid
	 *
	 * @return	$nextValid	Wether the next entry is valid
	 */
	public function next () {
		// Default is not valid
		$nextValid = false;

		// Is the result valid?
		if ($this->resultArray['status'] === "ok") {
			// The status is fine so let's have a look for the next entry
			if ($this->valid()) {
				// Next entry found, so count one up and cache it
				$this->currentPos++;
				$this->currentRow = $this->resultArray['rows'][$this->currentPos];
				$nextValid = true;
			} // END - if
		} // END - if

		// Return the result
		return $nextValid;
	}

	/**
	 * Seeks for to a specified position
	 *
	 * @param	$index	Index to seek for
	 * @return	void
	 */
	public function seek ($index) {
		// Rewind to beginning
		$this->rewind();

		// Search for the entry
		while ($this->currentPos < $index && ($this->valid())) {
			// Continue on
			$this->next();
		} // END - while
	}

	/**
	 * Gives back the current position or null if not found
	 *
	 * @return	$current	Current element to give back
	 */
	public function current () {
		// Default is not found
		$current = null;

		// Does the current enty exist?
		if (isset($this->resultArray['rows'][$this->currentPos])) {
			// Then get it
			$current = $this->resultArray['rows'][$this->currentPos];
		} // END - if

		// Return the result
		return $current;
	}

	/**
	 * Checks if next() and rewind will give a valid result
	 *
	 * @return	$isValid Wether the next/rewind entry is valid
	 */
	public function valid () {
		// By default nothing is valid
		$isValid = false;

		// Check if 
		if ((isset($this->resultArray['rows'][($this->currentPos + 1)])) && (isset($this->resultArray['rows'][0]))) {
			// All fine!
			$isValid = true;
		} // END - if

		// Return the result
		return $isValid;
	}

	/**
	 * Gets the current key of iteration
	 *
	 * @return	$currentPos	Key from iterator
	 */
	public function key () {
		return $this->currentPos;
	}

	/**
	 * Rewind to the beginning
	 *
	 * @return	void
	 */
	public function rewind () {
		$this->currentPos = -1;
	}

	/**
	 * Searches for an entry in the data result and returns it
	 *
	 * @param	$criteriaInstance	The criteria to look inside the data set
	 * @return	$result				Found result entry
	 */
	public function searchEntry (LocalSearchCriteria $criteriaInstance) {
		die("OK");
	}
}

// [EOF]
?>
