<?php
/**
 * The own template engine for loading caching and sending out the web pages
 * and emails.
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright(c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class TemplateEngine extends BaseFrameworkSystem implements CompileableTemplate {
	/**
	 * The local path name where all templates and sub folders for special
	 * templates are stored. We will internally determine the language plus
	 * "html" for web templates or "emails" for email templates
	 */
	private $basePath = "";

	/**
	 * The extension for web and email templates (not compiled templates)
	 */
	private $templateExtension = ".tpl";

	/**
	 * The extension for code templates (not compiled templates)
	 */
	private $codeExtension = ".ctp";

	/**
	 * Path relative to $basePath and language code for compiled code-templates
	 */
	private $compileOutputPath = "templates/_compiled";

	/**
	 * The raw (maybe uncompiled) template
	 */
	private $rawTemplateData = "";

	/**
	 * Template data with compiled-in variables
	 */
	private $compiledData = "";

	/**
	 * The last loaded template's FQFN for debugging the engine
	 */
	private $lastTemplate = "";

	/**
	 * The variable stack for the templates. This must be initialized and
	 * shall become an instance of FrameworkArrayObject.
	 */
	private $varStack = null;

	/**
	 * Configuration variables in a simple array
	 */
	private $configVariables = array();

	/**
	 * Loaded templates for recursive protection and detection
	 */
	private $loadedTemplates = array();

	/**
	 * Compiled templates for recursive protection and detection
	 */
	private $compiledTemplates = array();

	/**
	 * Loaded raw template data
	 */
	private $loadedRawData = null;

	/**
	 * Raw templates which are linked in code templates
	 */
	private $rawTemplates = null;

	/**
	 * A regular expression for variable=value pairs
	 */
	private $regExpVarValue = '/([\w_]+)(="([^"]*)"|=([\w_]+))?/';

	/**
	 * A regular expression for filtering out code tags
	 *
	 * E.g.: {?template:variable=value;var2=value2;[...]?}
	 */
	private $regExpCodeTags = '/\{\?([a-z_]+)(:("[^"]+"|[^?}]+)+)?\?\}/';

	/**
	 * Loaded helpers
	 */
	private $helpers = array();

	// Exception codes for the template engine
	const EXCEPTION_TEMPLATE_TYPE_IS_UNEXPECTED   = 0x020;
	const EXCEPTION_TEMPLATE_CONTAINS_INVALID_VAR = 0x021;
	const EXCEPTION_INVALID_VIEW_HELPER           = 0x022;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);

		// Set part description
		$this->setObjectDescription("Web template engine");

		// Create unique ID number
		$this->generateUniqueId();

		// Clean up a little
		$this->removeNumberFormaters();
		$this->removeSystemArray();
	}

	/**
	 * Creates an instance of the class TemplateEngine and prepares it for usage
	 *
	 * @param	$basePath		The local base path for all templates
	 * @param	$langInstance	An instance of LanguageSystem (default)
	 * @param	$ioInstance		An instance of FileIoHandler (default, middleware!)
	 * @return	$tplInstance	An instance of TemplateEngine
	 * @throws	BasePathIsEmptyException		If the provided $basePath is empty
	 * @throws	InvalidBasePathStringException	If $basePath is no string
	 * @throws	BasePathIsNoDirectoryException	If $basePath is no
	 *											directory or not found
	 * @throws	BasePathReadProtectedException	If $basePath is
	 *											read-protected
	 */
	public final static function createTemplateEngine ($basePath, ManageableLanguage  $langInstance, FileIoHandler $ioInstance) {
		// Get a new instance
		$tplInstance = new TemplateEngine();

		// Is the base path valid?
		if (empty($basePath)) {
			// Base path is empty
			throw new BasePathIsEmptyException($tplInstance, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!is_string($basePath)) {
			// Is not a string
			throw new InvalidBasePathStringException(array($tplInstance, $basePath), self::EXCEPTION_INVALID_STRING);
		} elseif (!is_dir($basePath)) {
			// Is not a path
			throw new BasePathIsNoDirectoryException(array($tplInstance, $basePath), self::EXCEPTION_INVALID_PATH_NAME);
		} elseif (!is_readable($basePath)) {
			// Is not readable
			throw new BasePathReadProtectedException(array($tplInstance, $basePath), self::EXCEPTION_READ_PROTECED_PATH);
		}

		// Get configuration instance
		$cfgInstance = FrameworkConfiguration::getInstance();

		// Set the base path
		$tplInstance->setBasePath($basePath);

		// Initialize the variable stack
		$tplInstance->initVariableStack();

		// Set the language and IO instances
		$tplInstance->setLanguageInstance($langInstance);
		$tplInstance->setFileIoInstance($ioInstance);

		// Set template extensions
		$tplInstance->setRawTemplateExtension($cfgInstance->readConfig('raw_template_extension'));
		$tplInstance->setCodeTemplateExtension($cfgInstance->readConfig('code_template_extension'));

		// Absolute output path for compiled templates
		$tplInstance->setCompileOutputPath(PATH . $cfgInstance->readConfig('compile_output_path'));

		// Return the prepared instance
		return $tplInstance;
	}

	/**
	 * Search for a variable in the stack
	 *
	 * @param		$var		The variable we are looking for
	 * @return	$idx		FALSE means not found, >=0 means found on a specific index
	 */
	private function isVariableAlreadySet ($var) {
		// First everything is not found
		$found = false;

		// Now search for it
		for ($idx = $this->varStack->getIterator(); $idx->valid(); $idx->next()) {
			// Get current item
			$currEntry = $idx->current();

			// Is the entry found?
			if ($currEntry['name'] == $var) {
				// Found!
				$found = $idx->key();
				break;
			}
		}

		// Return the current position
		return $found;
	}

	/**
	 * Return a content of a variable or null if not found
	 *
	 * @param	$var		The variable we are looking for
	 * @return	$content	Content of the variable or null if not found
	 */
	private function readVariable ($var) {
		// First everything is not found
		$content = null;

		// Now search for it
		for ($idx = $this->varStack->getIterator(); $idx->valid(); $idx->next()) {
			// Get current item
			$currEntry = $idx->current();

			// Is the entry found?
			if ($currEntry['name'] == $var) {
				// Found!
				$content = $currEntry['value'];
				break;
			}
		}

		// Return the current position
		return $content;
	}

	/**
	 * Add a variable to the stack
	 *
	 * @param		$var		The variable we are looking for
	 * @param		$value	The value we want to store in the variable
	 * @return	void
	 */
	private function addVariable ($var, $value) {
		// Add it to the stack
		$this->varStack->append(array(
			'name'  => trim($var),
			'value' => trim($value)
		));
	}

	/**
	 * Modify an entry on the stack
	 *
	 * @param		$var		The variable we are looking for
	 * @param		$value	The value we want to store in the variable
	 * @return	void
	 */
	private function modifyVariable ($var, $value) {
		// It should be there so let's look again...
		for ($idx = $this->varStack->getIterator(); $idx->valid(); $idx->next()) {
			// Get current entry
			$currEntry = $idx->current();

			// Is this the requested variable?
			if ($currEntry['name'] == $var) {
				// Change it to the other value
				$this->varStack->offsetSet($idx->key(), array(
					'name'  => $var,
					'value' => $value
				));
			}
		}
	}

	/**
	 * Setter for template type. Only "html", "emails" and "compiled" should
	 * be sent here
	 *
	 * @param	$templateType	The current template's type
	 * @return	void
	 */
	private final function setTemplateType ($templateType) {
		// Cast it
		$templateType = (string) $templateType;

		// And set it (only 2 letters)
		$this->templateType = $templateType;
	}

	/**
	 * Setter for the last loaded template's FQFN
	 *
	 * @param	$template	The last loaded template
	 * @return	void
	 */
	private final function setLastTemplate ($template) {
		// Cast it to string
		$template = (string) $template;
		$this->lastTemplate = $template;
	}

	/**
	 * Getter for the last loaded template's FQFN
	 *
	 * @return	$template	The last loaded template
	 */
	private final function getLastTemplate () {
		return $this->lastTemplate;
	}

	/**
	 * Assign a given congfiguration variable with a value
	 *
	 * @param	$var	The configuration variable we want to assign
	 * @return	void
	 */
	private final function assignConfigVariable ($var) {
		// Sweet and simple...
		$this->configVariables[$var] = $this->getConfigInstance()->readConfig($var);
	}

	/**
	 * Initialize the variable stack. This holds all variables for later
	 * compilation.
	 *
	 * @return	void
	 */
	public final function initVariableStack () {
		$this->varStack = new FrameworkArrayObject("FakedVariableStack");
	}

	/**
	 * Setter for base path
	 *
	 * @param		$basePath		The local base path for all templates
	 * @return	void
	 */
	public final function setBasePath ($basePath) {
		// Cast it
		$basePath = (string) $basePath;

		// And set it
		$this->basePath = $basePath;
	}

	/**
	 * Getter for base path
	 *
	 * @return	$basePath		The local base path for all templates
	 */
	public final function getBasePath () {
		// And set it
		return $this->basePath;
	}

	/**
	 * Setter for template extension
	 *
	 * @param		$templateExtension	The file extension for all uncompiled
	 *							templates
	 * @return	void
	 */
	public final function setRawTemplateExtension ($templateExtension) {
		// Cast it
		$templateExtension = (string) $templateExtension;

		// And set it
		$this->templateExtension = $templateExtension;
	}

	/**
	 * Setter for code template extension
	 *
	 * @param		$codeExtension		The file extension for all uncompiled
	 *							templates
	 * @return	void
	 */
	public final function setCodeTemplateExtension ($codeExtension) {
		// Cast it
		$codeExtension = (string) $codeExtension;

		// And set it
		$this->codeExtension = $codeExtension;
	}

	/**
	 * Getter for template extension
	 *
	 * @return	$templateExtension	The file extension for all uncompiled
	 *							templates
	 */
	public final function getRawTemplateExtension () {
		// And set it
		return $this->templateExtension;
	}

	/**
	 * Getter for code-template extension
	 *
	 * @return	$codeExtension		The file extension for all code-
	 *							templates
	 */
	public final function getCodeTemplateExtension () {
		// And set it
		return $this->codeExtension;
	}

	/**
	 * Setter for path of compiled templates
	 *
	 * @param	$compileOutputPath	The local base path for all compiled
	 *								templates
	 * @return	void
	 */
	public final function setCompileOutputPath ($compileOutputPath) {
		// Cast it
		$compileOutputPath = (string) $compileOutputPath;

		// And set it
		$this->compileOutputPath = $compileOutputPath;
	}

	/**
	 * Getter for template type
	 *
	 * @return	$templateType	The current template's type
	 */
	public final function getTemplateType () {
		return $this->templateType;
	}

	/**
	 * Assign (add) a given variable with a value
	 *
	 * @param	$var	The variable we are looking for
	 * @param	$value	The value we want to store in the variable
	 * @return	void
	 */
	public final function assignVariable ($var, $value) {
		// First search for the variable if it was already added
		$idx = $this->isVariableAlreadySet($var);

		// Was it found?
		if ($idx === false) {
			// Add it to the stack
			$this->addVariable($var, $value);
		} elseif (!empty($value)) {
			// Modify the stack entry
			$this->modifyVariable($var, $value);
		}
	}

	/**
	 * Removes a given variable
	 *
	 * @param	$var	The variable we are looking for
	 * @return	void
	 */
	public final function removeVariable ($var) {
		// First search for the variable if it was already added
		$idx = $this->isVariableAlreadySet($var);

		// Was it found?
		if ($idx !== false) {
			// Remove this variable
			$this->varStack->offsetUnset($idx);
		}
	}

	/**
	 * Private setter for raw template data
	 *
	 * @param	$rawTemplateData	The raw data from the template
	 * @return	void
	 */
	private final function setRawTemplateData ($rawTemplateData) {
		// Cast it to string
		$rawTemplateData = (string) $rawTemplateData;

		// And store it in this class
		$this->rawTemplateData = $rawTemplateData;
	}

	/**
	 * Private setter for compiled templates
	 *
	 * @return	void
	 */
	private final function setCompiledData ($compiledData) {
		// Cast it to string
		$compiledData = (string) $compiledData;

		// And store it in this class
		$this->compiledData = $compiledData;
	}

	/**
	 * Private loader for all template types
	 *
	 * @param	$template	The template we shall load
	 * @return	void
	 */
	private final function loadTemplate ($template) {
		// Cast it to string
		$template = (string) $template;

		// Get extension for the template
		$ext = $this->getRawTemplateExtension();

		// If we shall load a code-template we need to switch the file extension
		if ($this->getTemplateType() == $this->getConfigInstance()->readConfig('code_template_type')) {
			// Switch over to the code-template extension
			$ext = $this->getCodeTemplateExtension();
		}

		// Construct the FQFN for the template by honoring the current language
		$fqfn = sprintf("%s%s/%s/%s%s",
			$this->getBasePath(),
			$this->getLanguageInstance()->getLanguageCode(),
			$this->getTemplateType(),
			$template,
			$ext
		);

		// Load the raw template data
		$this->loadRawTemplateData($fqfn);
	}

	/**
	 * A private loader for raw template names
	 *
	 * @param	$fqfn	The full-qualified file name for a template
	 * @return	void
	 * @throws	NullPointerException	If $inputInstance is null
	 * @throws	NoObjectException		If $inputInstance is not an object
	 * @throws	MissingMethodException	If $inputInstance is missing a
	 *									required method
	 */
	private function loadRawTemplateData ($fqfn) {
		// Get a input/output instance from the middleware
		$ioInstance = $this->getFileIoInstance();

		// Validate the instance
		if (is_null($ioInstance)) {
			// Throw exception
			throw new NullPointerException($this, self::EXCEPTION_IS_NULL_POINTER);
		} elseif (!is_object($ioInstance)) {
			// Throw another exception
			throw new NoObjectException($ioInstance, self::EXCEPTION_IS_NO_OBJECT);
		} elseif (!method_exists($ioInstance, 'loadFileContents')) {
			// Throw yet another exception
			throw new MissingMethodException(array($ioInstance, 'loadFileContents'), self::EXCEPTION_MISSING_METHOD);
		}

		// Load the raw template
		$rawTemplateData = $ioInstance->loadFileContents($fqfn);

		// Store the template's contents into this class
		$this->setRawTemplateData($rawTemplateData);

		// Remember the template's FQFN
		$this->setLastTemplate($fqfn);
	}

	/**
	 * Try to assign an extracted template variable as a "content" or "config"
	 * variable.
	 *
	 * @param	$varName	The variable's name (shall be content orconfig) by
	 *						default
	 * @param	$var		The variable we want to assign
	 */
	private function assignTemplateVariable ($varName, $var) {
		// Is it not a config variable?
		if ($varName != "config") {
			// Regular template variables
			$this->assignVariable($var, "");
		} else {
			// Configuration variables
			$this->assignConfigVariable($var);
		}
	}

	/**
	 * Extract variables from a given raw data stream
	 *
	 * @param	$rawData	The raw template data we shall analyze
	 * @return	void
	 */
	private function extractVariablesFromRawData ($rawData) {
		// Cast to string
		$rawData = (string) $rawData;

		// Search for variables
		@preg_match_all('/\$(\w+)(\[(\w+)\])?/', $rawData, $variableMatches);

		// Did we find some variables?
		if ((is_array($variableMatches)) && (count($variableMatches) == 4) && (count($variableMatches[0]) > 0)) {
			// Initialize all missing variables
			foreach ($variableMatches[3] as $key=>$var) {
				// Try to assign it, empty strings are being ignored
				$this->assignTemplateVariable($variableMatches[1][$key], $var);
			}
		}
	}

	/**
	 * Main analysis of the loaded template
	 *
	 * @param	$templateMatches	Found template place-holders, see below
	 * @return	void
	 *
	 *---------------------------------
	 * Structure of $templateMatches:
	 *---------------------------------
	 * [0] => Array - An array with all full matches
	 * [1] => Array - An array with left part (before the ":") of a match
	 * [2] => Array - An array with right part of a match including ":"
	 * [3] => Array - An array with right part of a match excluding ":"
	 */
	private function analyzeTemplate (array $templateMatches) {
		// Backup raw template data
		$backup = $this->getRawTemplateData();

		// Initialize some arrays
		if (is_null($this->loadedRawData)) { $this->loadedRawData = array(); $this->rawTemplates = array(); }

		// Load all requested templates
		foreach ($templateMatches[1] as $template) {

			// Load and compile only templates which we have not yet loaded
			// RECURSIVE PROTECTION! BE CAREFUL HERE!
			if ((!isset($this->loadedRawData[$template])) && (!in_array($template, $this->loadedTemplates))) {

				// Template not found, but maybe variable assigned?
				if ($this->isVariableAlreadySet($template) !== false) {
					// Use that content here
					$this->loadedRawData[$template] = $this->readVariable($template);

					// Recursive protection:
					$this->loadedTemplates[] = $template;
				} else {
					// Then try to search for code-templates
					try {
						// Load the code template and remember it's contents
						$this->loadCodeTemplate($template);
						$this->loadedRawData[$template] = $this->getRawTemplateData();

						// Remember this template for recursion detection
						// RECURSIVE PROTECTION!
						$this->loadedTemplates[] = $template;
					} catch (FilePointerNotOpenedException $e) {
						// Even this is not done... :/
						$this->rawTemplates[] = $template;
					}
				}

			} // if ((!isset( ...

		} // for ($templateMatches ...

		// Restore the raw template data
		$this->setRawTemplateData($backup);
	}

	/**
	 * Compile a given raw template code and remember it for later usage
	 *
	 * @param	$code		The raw template code
	 * @param	$template	The template's name
	 * @return	void
	 */
	private function compileCode ($code, $template) {
		// Is this template already compiled?
		if (in_array($template, $this->compiledTemplates)) {
			// Abort here...
			return;
		}

		// Remember this template being compiled
		$this->compiledTemplates[] = $template;

		// Compile the loaded code in five steps:
		//
		// 1. Backup current template data
		$backup = $this->getRawTemplateData();

		// 2. Set the current template's raw data as the new content
		$this->setRawTemplateData($code);

		// 3. Compile the template data
		$this->compileTemplate();

		// 4. Remember it's contents
		$this->loadedRawData[$template] = $this->getRawTemplateData();

		// 5. Restore the previous raw content from backup variable
		$this->setRawTemplateData($backup);
	}

	/**
	 * Insert all given and loaded templates by running through all loaded
	 * codes and searching for their place-holder in the main template
	 *
	 * @param	$templateMatches	See method analyzeTemplate()
	 * @return	void
	 */
	private function insertAllTemplates (array $templateMatches) {
		// Run through all loaded codes
		foreach ($this->loadedRawData as $template=>$code) {

			// Search for the template
			$foundIndex = array_search($template, $templateMatches[1]);

			// Lookup the matching template replacement
			if (($foundIndex !== false) && (isset($templateMatches[0][$foundIndex]))) {

				// Get the current raw template
				$rawData = $this->getRawTemplateData();

				// Replace the space holder with the template code
				$rawData = str_replace($templateMatches[0][$foundIndex], $code, $rawData);

				// Set the new raw data
				$this->setRawTemplateData($rawData);

			} // END - if

		} // END - foreach
	}

	/**
	 * Load all extra raw templates
	 *
	 * @return	void
	 */
	private function loadExtraRawTemplates () {
		// Are there some raw templates we need to load?
		if (count($this->rawTemplates) > 0) {
			// Try to load all raw templates
			foreach ($this->rawTemplates as $key => $template) {
				try {
					// Load the template
					$this->loadWebTemplate($template);

					// Remember it's contents
					$this->rawTemplates[$template] = $this->getRawTemplateData();

					// Remove it from the loader list
					unset($this->rawTemplates[$key]);

					// Remember this template for recursion detection
					// RECURSIVE PROTECTION!
					$this->loadedTemplates[] = $template;
				} catch (FilePointerNotOpenedException $e) {
					// This template was never found. We silently ignore it
					unset($this->rawTemplates[$key]);
				}
			}
		}
	}

	/**
	 * Assign all found template variables
	 *
	 * @param		$varMatches	An array full of variable/value pairs.
	 * @return	void
	 */
	private function assignAllVariables (array $varMatches) {
		// Search for all variables
		foreach ($varMatches[1] as $key=>$var) {

			// Detect leading equals
			if (substr($varMatches[2][$key], 0, 1) == "=") {
				// Remove and cast it
				$varMatches[2][$key] = (string) substr($varMatches[2][$key], 1);
			}

			// Do we have some quotes left and right side? Then it is free text
			if ((substr($varMatches[2][$key], 0, 1) == "\"") && (substr($varMatches[2][$key], -1, 1) == "\"")) {
				// Free string detected! Which we can assign directly
				$this->assignVariable($var, $varMatches[3][$key]);
			} elseif (!empty($varMatches[2][$key])) {
				// Non-string found so we need some deeper analysis...
				/* @TODO Unfinished work or don't die here. */
				die("Deeper analysis not yet implemented!");
			}

		} // for ($varMatches ...
	}
	/**
	 * Compiles all loaded raw templates
	 *
	 * @param		$templateMatches	See method analyzeTemplate() for details
	 * @return	void
	 */
	private function compileRawTemplateData (array $templateMatches) {
		// Are some code-templates found which we need to compile?
		if (count($this->loadedRawData) > 0) {

			// Then compile all!
			foreach ($this->loadedRawData as $template=>$code) {

				// Is this template already compiled?
				if (in_array($template, $this->compiledTemplates)) {
					// Then skip it
					continue;
				}

				// Search for the template
				$foundIndex = array_search($template, $templateMatches[1]);

				// Lookup the matching variable data
				if (($foundIndex !== false) && (isset($templateMatches[3][$foundIndex]))) {

					// Split it up with another reg. exp. into variable=value pairs
					preg_match_all($this->regExpVarValue, $templateMatches[3][$foundIndex], $varMatches);

					// Assign all variables
					$this->assignAllVariables($varMatches);

				} // END - if (isset($templateMatches ...

				// Compile the loaded template
				$this->compileCode($code, $template);

			} // END - foreach ($this->loadedRawData ...

			// Insert all templates
			$this->insertAllTemplates($templateMatches);

		} // END - if (count($this->loadedRawData) ...
	}

	/**
	 * Inserts all raw templates into their respective variables
	 *
	 * @return	void
	 */
	private function insertRawTemplates () {
		// Load all templates
		foreach ($this->rawTemplates as $template=>$content) {
			// Set the template as a variable with the content
			$this->assignVariable($template, $content);
		}
	}

	/**
	 * Finalizes the compilation of all template variables
	 *
	 * @return	void
	 */
	private function finalizeVariableCompilation () {
		// Get the content
		$content = $this->getRawTemplateData();

		// Walk through all variables
		for ($idx = $this->varStack->getIterator(); $idx->valid(); $idx->next()) {
			// Get current entry
			$currEntry = $idx->current();

			// Replace all [$var] or {?$var?} with the content
			//* DEBUG: */ echo "name=".$currEntry['name'].", value=<pre>".htmlentities($currEntry['value'])."</pre>\n";
			$content = str_replace("\$content[".$currEntry['name']."]", $currEntry['value'], $content);
			$content = str_replace("[".$currEntry['name']."]", $currEntry['value'], $content);
			$content = str_replace("{?".$currEntry['name']."?}", $currEntry['value'], $content);
		} // END - for

		// Set the content back
		$this->setRawTemplateData($content);
	}

	/**
	 * Getter for raw template data
	 *
	 * @return	$rawTemplateData	The raw data from the template
	 */
	public final function getRawTemplateData () {
		return $this->rawTemplateData;
	}

	/**
	 * Getter for compiled templates
	 */
	public final function getCompiledData () {
		return $this->compiledData;
	}

	/**
	 * Load a specified web template into the engine
	 *
	 * @param		$template		The web template we shall load which is
	 *						located in "html" by default
	 * @return	void
	 */
	public final function loadWebTemplate ($template) {
		// Set template type
		$this->setTemplateType($this->getConfigInstance()->readConfig('web_template_type'));

		// Load the special template
		$this->loadTemplate($template);
	}

	/**
	 * Load a specified email template into the engine
	 *
	 * @param		$template		The email template we shall load which is
	 *						located in "emails" by default
	 * @return	void
	 */
	public final function loadEmailTemplate ($template) {
		// Set template type
		$this->setTemplateType($this->getConfigInstance()->readConfig('email_template_type'));

		// Load the special template
		$this->loadTemplate($template);
	}

	/**
	 * Load a specified code template into the engine
	 *
	 * @param		$template		The code template we shall load which is
	 *						located in "code" by default
	 * @return	void
	 */
	public final function loadCodeTemplate ($template) {
		// Set template type
		$this->setTemplateType($this->getConfigInstance()->readConfig('code_template_type'));

		// Load the special template
		$this->loadTemplate($template);
	}

	/**
	 * Compile all variables by inserting their respective values
	 *
	 * @return	void
	 */
	public final function compileVariables () {
		// Initialize the $content array
		$validVar = $this->getConfigInstance()->readConfig('tpl_valid_var');
		$dummy = array();

		// Iterate through all variables
		for ($idx = $this->varStack->getIterator(); $idx->valid(); $idx->next()) {

			// Get current variable from the stack
			$currVariable = $idx->current();

			// Transfer it's name/value combination to the $content array
			//* DEBUG: */ echo $currVariable['name']."=<pre>".htmlentities($currVariable['value'])."</pre>\n";
			$dummy[$currVariable['name']] = $currVariable['value'];

		}// END - if

		// Set the new variable (don't remove the second dollar !)
		$$validVar = $dummy;

		// Prepare all configuration variables
		$config = $this->configVariables;

		// Remove some variables
		unset($idx);
		unset($currVariable);

		// Run the compilation twice to get content from helper classes in
		$cnt = 0;
		while ($cnt < 3) {
			// Finalize the compilation of template variables
			$this->finalizeVariableCompilation();

			// Prepare the eval() command for comiling the template
			$eval = sprintf("\$result = \"%s\";",
				addslashes($this->getRawTemplateData())
			);

			// This loop does remove the backslashes (\) in PHP parameters
			/* @TODO Make this some nicer... */
			while (strpos($eval, "<?") !== false) {
				// Get left part before "<?"
				$evalLeft = substr($eval, 0, strpos($eval, "<?"));

				// Get all from right of "<?"
				$evalRight = substr($eval, (strpos($eval, "<?") + 2));

				// Is this a full PHP tag?
				if (substr(strtolower($evalRight), 0, 3) == "php") {
					// Remove "php" string from full PHP tag
					$evalRight = substr($evalRight, 3);
				} // END - if

				// Cut middle part out and remove escapes
				$evalMiddle = trim(substr($evalRight, 0, strpos($evalRight, "?>")));
				$evalMiddle = stripslashes($evalMiddle);

				// Remove the middle part from right one
				$evalRight = substr($evalRight, (strpos($evalRight, "?>") + 2));

				// And put all together
				$eval = sprintf("%s<%%php %s %%>%s", $evalLeft, $evalMiddle, $evalRight);
			} // END - while

			// Prepare PHP code for eval() command
			$eval = str_replace(
				"<%php", "\";",
				str_replace(
					"%>", "\$result .= \"", $eval
				)
			);

			// Debug message
			if ((defined('DEBUG_EVAL')) && (is_object($this->getDebugInstance()))) $this->getDebugInstance()->output(sprintf("[%s:] Constructed PHP command: <pre><em>%s</em></pre><br />\n",
				$this->__toString(),
				htmlentities($eval)
			));

			// Run the constructed command. This will "compile" all variables in
			eval($eval);

			// Set raw template data
			$this->setRawTemplateData($result);
			$cnt++;
		}

		// Set the new content
		$this->setCompiledData($result);
	}

	/**
	 * Compile all required templates into the current loaded one
	 *
	 * @return	void
	 * @throws	UnexpectedTemplateTypeException	If the template type is
	 *											not "code"
	 * @throws	InvalidArrayCountException		If an unexpected array
	 *											count has been found
	 */
	public final function compileTemplate () {
		// We will only work with template type "code" from configuration
		if ($this->getTemplateType() != $this->getConfigInstance()->readConfig('code_template_type')) {
			// Abort here
			throw new UnexpectedTemplateTypeException(array($this, $this->getTemplateType(), $this->getConfigInstance()->readConfig('code_template_type')), self::EXCEPTION_TEMPLATE_TYPE_IS_UNEXPECTED);
		} // END - if

		// Get the raw data.
		$rawData = $this->getRawTemplateData();

		// Remove double spaces and trim leading/trailing spaces
		$rawData = trim(str_replace("  ", " ", $rawData));

		// Search for raw variables
		$this->extractVariablesFromRawData($rawData);

		// Search for code-tags which are {? ?}
		preg_match_all($this->regExpCodeTags, $rawData, $templateMatches);

		// Analyze the matches array
		if ((is_array($templateMatches)) && (count($templateMatches) == 4) && (count($templateMatches[0]) > 0)) {
			// Entries are found:
			//
			// The main analysis
			$this->analyzeTemplate($templateMatches);

			// Compile raw template data
			$this->compileRawTemplateData($templateMatches);

			// Are there some raw templates left for loading?
			$this->loadExtraRawTemplates();

			// Are some raw templates found and loaded?
			if (count($this->rawTemplates) > 0) {

				// Insert all raw templates
				$this->insertRawTemplates();

				// Remove the raw template content as well
				$this->setRawTemplateData("");

			} // END - if

		} // END - if($templateMatches ...
	}

	/**
	 * Output the compiled page to the outside world. In case of web templates
	 * this would be vaild (X)HTML code. And in case of email templates this
	 * would store a prepared email body inside the template engine.
	 *
	 * @return	void
	 */
	public final function output () {
		// Check which type of template we have
		switch ($this->getTemplateType()) {
		case "html": // Raw HTML templates can be send to the output buffer
			// Quick-N-Dirty:
			$this->getWebOutputInstance()->output($this->getCompiledData());
			break;

		default: // Unknown type found
			// Construct message
			$msg = sprintf("[%s:] Unknown/unsupported template type <strong>%s</strong> detected.",
				$this->__toString(),
				$this->getTemplateType()
			);

			if ((is_object($this->getDebugInstance())) && (method_exists($this->getDebugInstance(), 'output'))) {
				// Use debug output handler
				$this->getDebugInstance()->output($msg);
				die();
			} else {
				// Put directly out
				// DO NOT REWRITE THIS TO app_die() !!!
				die($msg);
			}
			break;
		}
	}

	/**
	 * Loads a given view helper (by name)
	 *
	 * @param	$helperName	The helper's name
	 * @return	void
	 * @throws	ViewHelperNotFoundException	If the given view helper was not found
	 */
	protected function loadViewHelper ($helperName) {
		// Make first character upper case, rest low
		$helperName = ucfirst($helperName);

		// Is this view helper loaded?
		if (!isset($this->helpers[$helperName])) {
			// Create a class name
			$className = "{$helperName}ViewHelper";

			// Does this class exists?
			if (!class_exists($className)) {
				// Abort here!
				throw new ViewHelperNotFoundException(array($this, $helperName), self::EXCEPTION_INVALID_VIEW_HELPER);
			}

			// Generate new instance
			$eval = sprintf("\$this->helpers[%s] = %s::create%s();",
				$helperName,
				$className,
				$className
			);

			// Run the code
			eval($eval);
		}

		// Return the requested instance
		return $this->helpers[$helperName];
	}

	/**
	 * Assigns the last loaded raw template content with a given variable
	 *
	 * @param	$templateName	Name of the template we want to assign
	 * @param	$variableName	Name of the variable we want to assign
	 * @return	void
	 */
	public function assignTemplateWithVariable ($templateName, $variableName) {
		// Get the content from last loaded raw template
		$content = $this->getRawTemplateData();

		// Assign the variable
		$this->assignVariable($variableName, $content);

		// Purge raw content
		$this->setRawTemplateData("");
	}

	/**
	 * Transfers the content of this template engine to a given response instance
	 *
	 * @param	$responseInstance	An instance of a response class
	 * @return	void
	 */
	public function transferToResponse (Responseable $responseInstance) {
		// Get the content and set it in the response class
		$responseInstance->writeToBody($this->getCompiledData());
	}

	/**
	 * Assigns all the application data with template variables
	 *
	 * @param	$appInstance	A manageable application instance
	 * @return	void
	 */
	public function assignApplicationData (ManageableApplication $appInstance) {
		// Get long name and assign it
		$this->assignVariable("app_full_name" , $appInstance->getAppName());

		// Get short name and assign it
		$this->assignVariable("app_short_name", $appInstance->getAppShortName());

		// Get version number and assign it
		$this->assignVariable("app_version"   , $appInstance->getAppVersion());
	}
}

// [EOF]
?>
